# 3. Feladat - Hálózati kommunikáció - port szkenner 

## 3.a Feladat - Time szerver 

**Feladat:** Írjunk Python szkriptet `time_server.py` néven, ami megvalósít egy pontos időt visszaadó, 
TCP alapú szerver funkciót. A szerver a kliensek kapcsolódása után szimplán adja vissza a pontos időt 
(a klienseket külön szálon kezelve), majd bontsa a kapcsolatot.

A forráskód:

```python
#!/usr/bin/env python3

from socketserver import BaseRequestHandler, ThreadingTCPServer
import time
import argparse


class MyHandler(BaseRequestHandler):
    def handle(self):
        self.request.sendall(str.encode(time.ctime()))


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Time server")
    parser.add_argument('--ip', type=str)
    parser.add_argument('--port', type=int)
    args = parser.parse_args()
    a = vars(args)
    HOST, PORT = a["ip"], int(a["port"])
    server = ThreadingTCPServer((HOST, PORT), MyHandler)
    try:
        server.serve_forever()
    except KeyboardInterrupt:
        server.shutdown()
```

Futtatás:

```bash
$ chmod +x time_server.py
$ ./time_server.py --ip="localhost" --port=9999
```

Tesztelés:

```bash
$ nc localhost 9999
Mon Mar  7 17:56:58 2022
```

## 3.b Feladat - Port szkenner

**Feladat:** Valósítsunk meg egy port szkanner funkciót, amely egy megadott hoszton ("www.google.hu") 
végigszkenneli a "jól ismert" (well-known) portokat (1 - 1023). Eredményként írassuk ki a nyitott portokat. 

A megoldás:

```bash
$ pip3 install scapy
$ sudo python3
>>> from scapy.all import *
>>> packet = Ether() / IP(dst="www.google.hu") / TCP(dport=(1,1023))
>>> packet.show()
###[ Ethernet ]### 
  dst       = 00:00:0c:9f:f0:b6
  src       = 00:50:b6:8d:fb:1a
  type      = IPv4
###[ IP ]### 
     version   = 4
     ihl       = None
     tos       = 0x0
     len       = None
     id        = 1
     flags     = 
     frag      = 0
     ttl       = 64
     proto     = tcp
     chksum    = None
     src       = 152.66.182.33
     dst       = Net("www.google.hu/32")
     \options   \
###[ TCP ]### 
        sport     = ftp_data
        dport     = (1, 1023)
        seq       = 0
        ack       = 0
        dataofs   = None
        reserved  = 0
        flags     = S
        window    = 8192
        chksum    = None
        urgptr    = 0
        options   = ''
>>> ans,unans = srp(packet,timeout=1)
Begin emission:
Finished sending 1023 packets.
.......*..................*................................................
Received 251 packets, got 2 answers, remaining 1021 packets
>>> ans
<Results: TCP:2 UDP:0 ICMP:0 Other:0>
>>> for a in ans:
...     print(a[1][TCP].sport)
80
443
>>> 
```
